// Copyright (c) 2012 Ecma International.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
es5id: 8.7.2-3-1-s
description: eval - a property named 'eval' is permitted
---*/

  var o = { eval: 42};

reportCompare(0, 0);
