// Copyright 2009 the Sputnik authors.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
info: The Null Type has one value, called null
es5id: 8.2_A1_T1
description: Checking if execution of "var x = null" passes
---*/

//////////////////////////////////////////////////////////////////////////////
//CHECK#1
var x = null;
//
//////////////////////////////////////////////////////////////////////////////

reportCompare(0, 0);
