// Copyright 2009 the Sputnik authors.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
info: Check Var Statement for automatic semicolon insertion
es5id: 7.9_A7_T4
description: Checking if execution of "var \n x" passes
---*/

//CHECK#1
var
x

reportCompare(0, 0);
