// Generated by make_intl_data.py. DO NOT EDIT.
// Version: 2017-01-01

/**
 * Mapping from currency codes to the number of decimal digits used for them.
 * Default is 2 digits.
 *
 * Spec: ISO 4217 Currency and Funds Code List.
 * http://www.currency-iso.org/en/home/tables/table-a1.html
 */
var currencyDigits = {
    // Bahraini Dinar (BAHRAIN)
    BHD: 3,
    // Burundi Franc (BURUNDI)
    BIF: 0,
    // Unidad de Fomento (CHILE)
    CLF: 4,
    // Chilean Peso (CHILE)
    CLP: 0,
    // Djibouti Franc (DJIBOUTI)
    DJF: 0,
    // Guinea Franc (GUINEA)
    GNF: 0,
    // Iraqi Dinar (IRAQ)
    IQD: 3,
    // Iceland Krona (ICELAND)
    ISK: 0,
    // Jordanian Dinar (JORDAN)
    JOD: 3,
    // Yen (JAPAN)
    JPY: 0,
    // Comoro Franc (COMOROS (THE))
    KMF: 0,
    // Won (KOREA (THE REPUBLIC OF))
    KRW: 0,
    // Kuwaiti Dinar (KUWAIT)
    KWD: 3,
    // Libyan Dinar (LIBYA)
    LYD: 3,
    // Rial Omani (OMAN)
    OMR: 3,
    // Guarani (PARAGUAY)
    PYG: 0,
    // Rwanda Franc (RWANDA)
    RWF: 0,
    // Tunisian Dinar (TUNISIA)
    TND: 3,
    // Uganda Shilling (UGANDA)
    UGX: 0,
    // Uruguay Peso en Unidades Indexadas (URUIURUI) (URUGUAY)
    UYI: 0,
    // Dong (VIET NAM)
    VND: 0,
    // Vatu (VANUATU)
    VUV: 0,
    // CFA Franc BEAC (CAMEROON)
    // CFA Franc BEAC (CENTRAL AFRICAN REPUBLIC (THE))
    // CFA Franc BEAC (CHAD)
    // CFA Franc BEAC (CONGO (THE))
    // CFA Franc BEAC (EQUATORIAL GUINEA)
    // CFA Franc BEAC (GABON)
    XAF: 0,
    // CFA Franc BCEAO (BENIN)
    // CFA Franc BCEAO (BURKINA FASO)
    // CFA Franc BCEAO (CÔTE D'IVOIRE)
    // CFA Franc BCEAO (GUINEA-BISSAU)
    // CFA Franc BCEAO (MALI)
    // CFA Franc BCEAO (NIGER (THE))
    // CFA Franc BCEAO (SENEGAL)
    // CFA Franc BCEAO (TOGO)
    XOF: 0,
    // CFP Franc (FRENCH POLYNESIA)
    // CFP Franc (NEW CALEDONIA)
    // CFP Franc (WALLIS AND FUTUNA)
    XPF: 0,
};
