// |reftest| error:SyntaxError
// Copyright 2009 the Sputnik authors.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
info: "RegularExpressionFirstChar :: * or \\ or / or [empty] is incorrect"
es5id: 7.8.5_A1.2_T2
description: \
negative:
  phase: early
  type: SyntaxError
---*/

//CHECK#1
/\/
