// Copyright 2009 the Sputnik authors.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
info: Operator x-- returns ToNumber(x)
es5id: 11.3.2_A4_T2
description: Type(x) is number primitive or Number object
---*/

//CHECK#1
var x = 1.1;
var y = x--;
if (y !== 1.1) {
  $ERROR('#1: var x = 1.1; var y = x--; y === 1.1. Actual: ' + (y));
}

//CHECK#2
var x = new Number(-0.1);
var y = x--;
if (y !== -0.1) {
  $ERROR('#2: var x = new Number(-0.1); var y = x--; y === -0.1. Actual: ' + (y));
}

reportCompare(0, 0);
