// This file was procedurally generated from the following sources:
// - src/dstr-binding/ary-ptrn-rest-id-exhausted.case
// - src/dstr-binding/default/try.template
/*---
description: RestElement applied to an exhausted iterator (try statement)
esid: sec-runtime-semantics-catchclauseevaluation
es6id: 13.15.7
features: [Symbol.iterator, destructuring-binding]
flags: [generated]
info: |
    Catch : catch ( CatchParameter ) Block

    [...]
    5. Let status be the result of performing BindingInitialization for
       CatchParameter passing thrownValue and catchEnv as arguments.
    [...]

    13.3.3.6 Runtime Semantics: IteratorBindingInitialization
    BindingRestElement : ... BindingIdentifier
    1. Let lhs be ResolveBinding(StringValue of BindingIdentifier,
       environment).
    2. ReturnIfAbrupt(lhs). 3. Let A be ArrayCreate(0). 4. Let n=0. 5. Repeat,
       [...]
       b. If iteratorRecord.[[done]] is true, then
          i. If environment is undefined, return PutValue(lhs, A).
          ii. Return InitializeReferencedBinding(lhs, A).

---*/

var ranCatch = false;

try {
  throw [1, 2];
} catch ([, , ...x]) {
  assert(Array.isArray(x));
  assert.sameValue(x.length, 0);
  ranCatch = true;
}

assert(ranCatch, 'executed `catch` block');

reportCompare(0, 0);
