// Copyright 2016 Microsoft, Inc. All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
author: Brian Terlson <brian.terlson@microsoft.com>
esid: pending
description: >
  Async function expressions return promises
---*/

var p = async function() { }();
assert(p instanceof Promise, "async functions return promise instances");

reportCompare(0, 0);
