// file: tco-helper.js
// This defines the number of consecutive recursive function calls that must be
// made in order to prove that stack frames are properly destroyed according to
// ES2015 tail call optimization semantics.
var $MAX_ITERATIONS = 100000;
